/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#ifndef JOBSETUPWINDOW_H
#define JOBSETUPWINDOW_H
#include <limits.h>
#include <Window.h>
#include <Message.h>
#include <OS.h>
#include <Box.h>
#include <TextControl.h>
#include <CheckBox.h>
#include <RadioButton.h>
#include <TextView.h>
#include <PopUpMenu.h>
#include <MenuField.h>
#include <MenuItem.h>

#include <Slider.h>
#include <TabView.h>
#include <Button.h>
#include <stdio.h>

#include <PrinterAddon.h>
#include <PageDefs.h>

#include "PageOrderView.h"
#include "PagePrintStyleView.h"
#include "PrintingQualityView.h"
#include "PrintingModeView.h"
#include "ResolutionView.h"
#include "GUIUtils.h"
#include "InkLevelView.h"
namespace JobSetupWindowSpace
{
  const int32 DEF_WIDTH = 376;
  const int32 DEF_HEIGHT = 288;
  
  const char LB_GROUP_PAGES[]				= "Pages";
  const char LB_ALL_PAGE[]					= "All pages";
  const char LB_SEL_PAGE[]					= "Page";
    
  const char LB_TAB_PRINTJOB[] 				= "Print";
  const char LB_TAB_OPTION[]				= "Options";
  const char LB_TAB_UTILITIES[]				= "Utilities";
  
  const char LB_DISCARD[]					= "Discard";
  const char LB_PREVIEW[]					= "Preview";
  const char LB_PRINT[]						= "Print";
  const char LB_TO[]						= "To";
  
  const char LB_GROUP_COPIES[]				= "Copies";
  const char LB_COPIES[]					= "Copies: ";
  const char LB_COLLATE[]					= "Collate copies";
  const char LB_REVERSE[]					= "Reverse order";
  
  const char LB_PAPERTYPE_GROUP[]			= "Paper";
  const char LB_PAPERTYPE[]					= "Paper: ";
  
  const char LB_MODE_COLOR_GROUP[]			= "Color";
  
  const char LB_MODE_COLOR_COLOR[]			= "Color";
  const char LB_MODE_COLOR_MONO[]			= "Monochrome";

  const char LB_QUALITY_GROUP[]				= "Mode";
  const char LB_QUALITY[]					= "Quality: ";
  const char LB_MIN_QUALITY[]				= "Poor";
  const char LB_MAX_QUALITY[]				= "Best";
  
  const char LB_SNOOZE_CHECK[]				= "Snooze Check";
  const char LB_SNOOZE_CHECK_DETAIL[]		= "Snooze Check";
  
  const char LB_SNOOZE_CLEAN[]				= "Snooze Clean";
  const char LB_SNOOZE_CLEAN_DETAIL[]		= "Snooze Check";

  const char LB_EJECT_INK[]					= "Eject Ink";
  const char LB_EJECT_INK_DETAIL[]			= "Eject Ink";

  const char LB_INK_BOX[]					= "Ink Level";
  const char LB_INK_LEVEL[]					= "Ink Level";
  const char LB_UPDATE_INKLEVEL[]			= "Update Ink level";
  
  const uint32 MSGCODE_DISCARD  			= 'msg0' + 0;
  const uint32 MSGCODE_PREVIEW				= 'msg0' + 1;
  const uint32 MSGCODE_PRINT				= 'msg0' + 2;
  const uint32 MSGCODE_PAGE_ALL_PAGE		= 'msg0' + 3;
  const uint32 MSGCODE_PAGE_SEL_PAGE		= 'msg0' + 4;
  const uint32 MSGCODE_PAGE_SEL_START		= 'msg0' + 5;
  const uint32 MSGCODE_PAGE_SEL_END			= 'msg0' + 6;
  const uint32 MSGCODE_NUM_OF_COPIES		= 'msg0' + 7;
  const uint32 MSGCODE_COLLATE_ORDER		= 'msg0' + 8;
  const uint32 MSGCODE_REVERSE_ORDER		= 'msg0' + 9;
  
  const uint32 MSGCODE_PAPERTYPE			= 'msg0' + 10;
  const uint32 MSGCODE_COLOR				= 'msg0' + 11;
  const uint32 MSGCODE_QUALITY_SCALE		= 'msg0' + 12;
  
  const uint32 MSGCODE_SNOOZE_CHECK			= 'msg0' + 13;
  const uint32 MSGCODE_SNOOZE_CLEAN			= 'msg0' + 14;
  const uint32 MSGCODE_EJECT_INK			= 'msg0' + 15;
  const uint32 MSGCODE_UPDATE_INKLEVEL		= 'msg0' + 16;
};
class JobSetupWindow : public BWindow
{
  public:
    JobSetupWindow(const char *,PrinterAddon *addon,PrinterInfo *printer_info,BMessage *msg);
    virtual void Quit();
    virtual void MessageReceived(BMessage *);
    void Go();
  private:
    void BuildGUI();				// yes
    void SetTo(BMessage *);			// yes
    void Retrieve(BMessage *);		// yes
    void SetPreview(bool);			// yes
  private:
    // event  
    void OnBtnPrint();				// yes
    void OnBtnPreview();			// yes
    void OnBtnDiscard();			// yes
    void OnBtnSnoozeClean();		// yes
    void OnBtnSnoozeCheck();		// yes
    void OnBtnEjectInk();			// yes
    void OnBtnUpdateInkLevel();		// yes
    void OnEventAllPage();			// yes
    void OnEventSelPage();			// yes
    void OnEventCopyNum();			// yes
    void OnEventCollateOrder();		// yes
    void OnEventReverseOrder();		// yes
    void OnEventPaperType();		// yes
    void OnEventColorMode();		// yes
    void OnEventQualityScale();		// yes
    // sub function
    void SetEnableSelPageField(bool);
    void UpdateInkLevelStatus(const char *);
    void UpdateFunctionAvailability();
  private: 
    // Addons   
    void LoadPaperType();						// yes
    bool IsGUIColorMode();						// yes
    int32 GetGUISelectPaperType();				// yes
    void SetSDQualityFromGUI();					// yes
    void CreateSDQuality(int value,int count);  // yes
    void SetLBQualityFromGUI();					// yes
    void SetBVQualityFromGUI();					// yes
    int32 GetNumberOfCopies();					// yes
  private:
    sem_id fsem;
    PrinterAddon *faddon;
    PrinterInfo  *fprinter_info;
    BMessage *fmsg;
    int funit;
  private:
    BTabView     *tb_view;
    /* ======================================= */
    // Print Job
    BRadioButton *rb_all_page;
    BRadioButton *rb_sel_page;
    BTextControl *tv_page_start;
    BTextControl *tv_page_end;
    // Copies
    PageOrderView *bv_copies;
    BTextControl  *tf_copies;
    
    BCheckBox    *chk_collate;
    BCheckBox    *chk_reverse;
    
    PagePrintStyleView *bv_style;
    /* ======================================= */
    /* ======================================= */
    // Option 
    BPopUpMenu			*mn_paper_type;
    PrintingModeView	*bv_color_mode;
    BRadioButton		*rb_mode_color;
    BRadioButton		*rb_mode_mono;
    BBox				*f_quality_box;
    BRect        		f_quality_rect;
    PrintingQualityView *bv_quality;
    BSlider				*sd_quality;
    ResolutionView		*lb_resolution;
    InkLevelView	    *inklevel;
    BStringView         *lb_snooze_check;
    BStringView         *lb_snooze_clean;
    BStringView         *lb_ink_eject;
    BStringView         *lb_ink_level;
    /* ======================================= */
    
    /* ======================================= */
    /* ======================================= */
    BButton		 *btn_discard;
    BButton		 *btn_preview;
    BButton		 *btn_print;
    
    BButton      *btn_snooze_check;
    BButton      *btn_snooze_clean;
    BButton      *btn_ink_eject;
    BButton      *btn_ink_level;
};
#endif