/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include <RasterSpool.h>
RasterSpool::RasterSpool()
:ScaleInfo()
{
  UnsetValue();
}
RasterSpool::~RasterSpool()
{
  Unset();
}

bool RasterSpool::InitOK()
{
  return finit;
}


BRect RasterSpool::PrintableFrame()
{
  return fprintableframe;
}

BRect RasterSpool::ScaningFrame()
{
  return BRect(fprintableframe.left + fscanframe.left,
               fprintableframe.top  + fscanframe.top,
               fprintableframe.left + fscanframe.right,
               fprintableframe.top  + fscanframe.bottom);
}

BRect RasterSpool::ScaningRect()
{
  return BRect(0,0,fscanframe.Width(),fscanframe.Height());
}

int RasterSpool::PageNumber()
{
  return fpagenumber;
}

void RasterSpool::UnsetValue()
{
  fpagenumber = 0;
  
  fprintableframe = RasterSpoolSpace::EMPTYRECT;
  fscanframe      = RasterSpoolSpace::EMPTYRECT;
  fpostscanframe  = RasterSpoolSpace::EMPTYRECT;
  fbufferrect	  = RasterSpoolSpace::EMPTYRECT;
  
  fspoolfile = NULL;
  fbuffer    = NULL;
  
  finit = false;
}
void RasterSpool::Uninit()
{
  if (fbuffer != NULL)
  delete fbuffer;
  UnsetValue();
  ScaleInfo::Unset();
}

/* ========================================================= */
/* Init(SpoolFile,maxline									 */
bool RasterSpool::Init(SpoolFile *spoolfile,int maxline)
{
  BRect nonscale_printable_rect_nonrotate;
  BRect nonscale_printable_rect_rotate;
  BRect scale_printable_rect_rotate;
  size_t buffersize;
  
  Unset();

  if (maxline < 1)
  return InitOK();
    
  fspoolfile = spoolfile;
  ScaleInfo::SetTo(fspoolfile->XResolution(),fspoolfile->YResolution());  
  
  // calculate printable rect and after rotate value
  nonscale_printable_rect_nonrotate = fspoolfile->PrintableRect();
  if (fspoolfile->Portrait())
  nonscale_printable_rect_rotate = nonscale_printable_rect_nonrotate;
  else
  nonscale_printable_rect_rotate = BRect(nonscale_printable_rect_nonrotate.top,
                                         nonscale_printable_rect_nonrotate.left,
                                         nonscale_printable_rect_nonrotate.bottom,
                                         nonscale_printable_rect_nonrotate.right);
  scale_printable_rect_rotate    = ScaleInfo::NonScaleToPrintScale(nonscale_printable_rect_rotate);

  // assign printable frame value to scale printable rect rotate
  fprintableframe = scale_printable_rect_rotate;

  // calculate buffer rect value
  if (maxline > fprintableframe.IntegerWidth())
  fbufferrect = BRect(0,0,fprintableframe.IntegerWidth(),fprintableframe.IntegerHeight());
  else
  fbufferrect = BRect(0,0,fprintableframe.IntegerWidth(),maxline);
  
  // calculate buffer size
  buffersize = fbufferrect.IntegerHeight() * fbufferrect.IntegerWidth();
  if (fspoolfile->ColorPrint())
  buffersize = buffersize * sizeof(uint32);

  // allocating memory for buffer
  fbuffer = new char[buffersize];
  if (fbuffer == NULL)
  {
    Uninit();
    return InitOK();
  }
  fpagenumber = 0;
  finit = true;
  return InitOK();
}
/* ========================================================= */

/* ========================================================= */
/* Unset													 */
void RasterSpool::Unset()
{
  if (InitOK())
  Uninit();
}
/* ========================================================= */

/* ========================================================= */
/* SetToPage												 */
bool RasterSpool::SetToPage(int pagenumber)
{
  if (!InitOK())
  return false;
  
  if ((pagenumber < 1) || (pagenumber > fspoolfile->PrintFileHeader().page_count))
  {
    fendofscan = true;
    return false;
  }

  fendofscan = false;
  fpagenumber = pagenumber;
  
  fscanframe     = RasterSpoolSpace::EMPTYRECT;
  fpostscanframe = fbufferrect;
  return true;
}
/* ========================================================= */

/* ========================================================= */
/* ScanNext													 */
bool RasterSpool::ScanNext()
{
  BPoint  origin_point;
  BRect   scan_rect;
  BRect   bitmap_rect;
  BRect   printscale_printable_rect;
  BRect   page_rect;
  BRect   rotate_scan_frame;
  
  BView   *view;
  BBitmap *bitmap;

  int     reminder_line;  
  if (fendofscan)
  return false;
  
  fscanframe = fpostscanframe;
  page_rect        = fspoolfile->PrintableRect();
  page_rect.top    = (floor(page_rect.Height()) * (PageNumber() - 1));
  page_rect.bottom = (floor(page_rect.Height()) * PageNumber());
  
  // calculate PrintScale Printable Rect Rect
  printscale_printable_rect = BRect(0,0,fprintableframe.Width(),fprintableframe.Height());

  // to calculate the actual drawing rect from the fscanframe.RECT() 
  scan_rect = BRect(0,0,fscanframe.Width(),fscanframe.Height());
  // calculate bitmap rect from scan rect  
  bitmap_rect = ScaleInfo::PrintScaleToDrawScale(scan_rect);
  
  // re-rotate bitmap rect
  if (fspoolfile->Landscape())
  bitmap_rect = BRect(0,0,bitmap_rect.Height(),bitmap_rect.Width());

  // create a bitmap size of bitmap rect
  bitmap = new BBitmap(bitmap_rect,B_RGBA32,true,false);
  // create a view size of bitmap rect
  view = new BView(bitmap_rect,"PartialSpoolView",B_FOLLOW_NONE,B_WILL_DRAW);
  
  // initial drawing sequence
  bitmap->Lock();
  bitmap->AddChild(view);

  // fill background with white color
  view->SetHighColor(255,255,255);
  view->FillRect(view->Bounds());
  
  BRegion *draw_region = new BRegion(bitmap_rect);
  view->ConstrainClippingRegion(draw_region);
  delete draw_region;
  
  if (fspoolfile->Rotate180())
  {
    if (fspoolfile->Portrait())
    rotate_scan_frame.OffsetTo(0,printscale_printable_rect.bottom - fscanframe.bottom);
    else
    rotate_scan_frame.OffsetTo(printscale_printable_rect.right - fscanframe.right,0);
  }
  else
  rotate_scan_frame = fscanframe;
 
  // calculate the origin point
  if (fspoolfile->Portrait())
  {
    origin_point.x = -ScaleInfo::PrintScaleXToDrawScale(rotate_scan_frame.left);
    origin_point.y = -ScaleInfo::PrintScaleYToDrawScale(rotate_scan_frame.top);
  }
  else
  {
    origin_point.x = -ScaleInfo::PrintScaleYToDrawScale(rotate_scan_frame.top);
    origin_point.y = -ScaleInfo::PrintScaleXToDrawScale(rotate_scan_frame.left);
  }
  origin_point.x -= ScaleInfo::NonScaleXToDrawScale(page_rect.left);
  origin_point.y -= ScaleInfo::NonScaleYToDrawScale(page_rect.top);    

  // Set view origin
  view->SetOrigin(origin_point);
  
  // Set view draw scale
  view->SetScale(ScaleInfo::DrawScale()); 
  
  // start drawing
  for (int i = 0;i < fspoolfile->CountPictures();i++)
  {
    PictureSpool *pic = fspoolfile->GetPicture(i);
    if (page_rect.Contains(pic->Frame()))
    {
      BPoint draw_offset;

      draw_offset.x = ScaleInfo::NonScaleXToDrawScale(pic->Frame().left);
      draw_offset.y = ScaleInfo::NonScaleYToDrawScale(pic->Frame().top);
      
      view->DrawPicture(pic->Picture(),draw_offset);
    }
  }
  
  // finalize drawing sequence
  bitmap->RemoveChild(view);
  bitmap->Unlock();
  delete view;
  
  // copy data from bitmap to fbuffer
  if (fspoolfile->Portrait())
  {
    if (fspoolfile->Rotate180())
    CopyData_PortraitRotate180(scan_rect,bitmap);
    else
    CopyData_Portrait(scan_rect,bitmap);
  }
  else
  {
    if (fspoolfile->Rotate180())
    CopyData_LandscapeRotate180(scan_rect,bitmap);
    else
    CopyData_Landscape(scan_rect,bitmap);
  }

  // delete bitmap
  delete bitmap;
  
  // update the fscanframe
  reminder_line = (int) (printscale_printable_rect.bottom - fscanframe.bottom);
  if (reminder_line <= 0)
  fendofscan = true;
  else if (reminder_line < fbufferrect.IntegerHeight())
  {
    fpostscanframe        = fbufferrect;
    fpostscanframe.top    = fscanframe.bottom;
    fpostscanframe.bottom = printscale_printable_rect.bottom;
  }
  else
  {
    fpostscanframe        = fbufferrect;
    fpostscanframe.OffsetTo(0,fscanframe.bottom);
  } 
  return true;  
}
/* ========================================================= */

void RasterSpool::CopyData_PortraitRotate180(BRect rect,BBitmap *bitmap)
{
  rgb_color color_value;
  if (fspoolfile->ColorPrint())
  {
    for (int j = 0;j < rect.IntegerHeight();j++)
    {
      for (int i = 0;i < rect.IntegerWidth();i++)
      {
        RetrievePixels(bitmap,rect.right - i,rect.bottom - j,&color_value);
        WriteColorPixels(i,j,color_value);      
      }
    }
  }
  else
  {
    for (int j = 0;j < rect.IntegerHeight();j++)
    {
      for (int i = 0;i < rect.IntegerWidth();i++)
      {
        RetrievePixels(bitmap,rect.right - i,rect.bottom - j,&color_value);
        WriteGrayPixels(i,j,color_value);      
      }
    }
  }
}
void RasterSpool::CopyData_Portrait(BRect rect,BBitmap *bitmap)
{
  rgb_color color_value;
  if (fspoolfile->ColorPrint())
  {
    for (int j = 0;j < rect.IntegerHeight();j++)
    {
      for (int i = 0;i < rect.IntegerWidth();i++)
      {
        RetrievePixels(bitmap,i,j,&color_value);
        WriteColorPixels(i,j,color_value);      
      }
    }
  }
  else
  {
    for (int j = 0;j < rect.IntegerHeight();j++)
    {
      for (int i = 0;i < rect.IntegerWidth();i++)
      {
        RetrievePixels(bitmap,i,j,&color_value);
        WriteGrayPixels(i,j,color_value);      
      }
    }
  }
}
    
void RasterSpool::CopyData_LandscapeRotate180(BRect rect,BBitmap *bitmap)
{
  rgb_color color_value;
  int32 bitmap_height = bitmap->Bounds().IntegerHeight();
  if (fspoolfile->ColorPrint())
  {
    for (int j = 0;j < rect.IntegerHeight();j++)
    {
      for (int i = 0;i < rect.IntegerWidth();i++)
      {
        RetrievePixels(bitmap,j,bitmap_height - i,&color_value);
        WriteColorPixels(i,j,color_value);      
      }
    }
  }
  else
  {
    for (int j = 0;j < rect.IntegerHeight();j++)
    {
      for (int i = 0;i < rect.IntegerWidth();i++)
      {
        RetrievePixels(bitmap,j,bitmap_height - i,&color_value);
        WriteGrayPixels(i,j,color_value);      
      }
    }
  }
}

void RasterSpool::CopyData_Landscape(BRect rect,BBitmap *bitmap)
{
  rgb_color color_value;
  int32 bitmap_width = bitmap->Bounds().IntegerWidth();
  if (fspoolfile->ColorPrint())
  {
    for (int j = 0;j < rect.IntegerHeight();j++)
    {
      for (int i = 0;i < rect.IntegerWidth();i++)
      {
        RetrievePixels(bitmap,bitmap_width - j,i,&color_value);
        WriteColorPixels(i,j,color_value);      
      }
    }
  }
  else
  {
    for (int j = 0;j < rect.IntegerHeight();j++)
    {
      for (int i = 0;i < rect.IntegerWidth();i++)
      {
        RetrievePixels(bitmap,bitmap_width - j,i,&color_value);
        WriteGrayPixels(i,j,color_value);      
      }
    }
  }
}

void RasterSpool::RetrievePixels(BBitmap *bitmap,int x,int y,rgb_color *value)
{
  *value = *((rgb_color *) bitmap->Bits() + ((y * bitmap->BytesPerRow()) + (x * sizeof(uint32)))); 
}
void RasterSpool::WriteColorPixels(int x,int y,rgb_color value)
{
  *((rgb_color *) (fbuffer + (((y * fbufferrect.IntegerWidth()) + x) * sizeof(uint32)))) = value; 
}
void RasterSpool::WriteGrayPixels(int x,int y,rgb_color value)
{
  *((uint8 *) (fbuffer + ((y * fbufferrect.IntegerWidth()) + x))) = (uint8) (value.red + value.green + value.blue) / 3;
}